"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = __importDefault(require("./logger"));
const site_manager_1 = __importDefault(require("./site-manager"));
const async_1 = __importDefault(require("async"));
const cheerio = __importStar(require("cheerio"));
const lodash_1 = __importDefault(require("lodash"));
const superagent_1 = __importDefault(require("superagent"));
const DEFAULT_OPTIONS = {
    retrySleepTime: 1000,
    retryStatusCodes: [503],
};
class PriceFinder {
    config;
    constructor(options = undefined) {
        logger_1.default.debug('initializing PriceFinder');
        logger_1.default.debug('user supplied options: %j', options);
        // merge options, taking the user supplied if duplicates exist
        this.config = lodash_1.default.extend(DEFAULT_OPTIONS, options);
        logger_1.default.debug('merged config: %j', this.config);
    }
    /**
     * Scrapes a website specified by the uri and finds the item price.
     *
     * @param  {string}   uri      The uri of the website to scan
     * @return {number | undefined} the item price if found
     * @throws Error if the scrape fails
     */
    async findItemPrice(uri) {
        logger_1.default.debug('findItemPrice with uri: %s', uri);
        let site;
        try {
            site = site_manager_1.default.loadSite(uri);
        }
        catch (error) {
            logger_1.default.error('error loading site: ', error);
            throw error;
        }
        logger_1.default.debug('scraping the page...');
        // page scrape the site to load the page data
        return new Promise((resolve, reject) => this.pageScrape(site, (err, pageData) => {
            if (err) {
                logger_1.default.error('error retrieving pageData: ', err);
                return reject(err);
            }
            logger_1.default.debug('pageData found, loading price from site...');
            // find the price on the website
            const price = site.findPriceOnPage(pageData);
            // error check
            if (price === -1) {
                logger_1.default.error('unable to find price');
                return reject(Error(`unable to find price for uri: ${uri}`));
            }
            logger_1.default.debug('price found, returning price: %s', price);
            // return the item price
            return resolve(price);
        }));
    }
    // ==================================================
    // ============== PRIVATE FUNCTIONS =================
    // ==================================================
    pageScrape(site, callback) {
        let pageData;
        async_1.default.whilst(
        // run until we get page data
        (cb) => cb(null, !pageData), 
        // hit the site to get the item details
        (whilstCallback) => {
            logger_1.default.debug('scraping uri: %s', site.getURIForPageData());
            superagent_1.default
                .get(site.getURIForPageData())
                .set('User-Agent', 'Mozilla')
                .set('Accept-Language', 'en-US')
                .set('Accept', 'text/html')
                // TODO any type
                // rome-ignore lint/suspicious/noExplicitAny: use any for now
                .end((err, response) => {
                if (err) {
                    return whilstCallback(err);
                }
                if (response) {
                    if (response.statusCode === 200) {
                        logger_1.default.debug('response statusCode is 200, retrieving pageData');
                        // load the text of the page through cheerio
                        pageData = cheerio.load(response.text);
                        return whilstCallback(null);
                    }
                    else if (this.config.retryStatusCodes.indexOf(response.statusCode) > -1) {
                        // if we get a statusCode that we should retry, try again
                        logger_1.default.debug('response status part of retryStatusCodes, status: %s, retrying...', response.statusCode);
                        logger_1.default.debug(`sleeping for: ${this.config.retrySleepTime}ms`);
                        return setTimeout(() => whilstCallback(), this.config.retrySleepTime);
                    }
                    else {
                        // else it's a bad response status, all stop
                        return whilstCallback(new Error(`response status: ${response.statusCode}`));
                    }
                }
                else {
                    return whilstCallback(new Error('no response object found!'));
                }
            });
        }, 
        // once we have the pageData, or error, return
        (err) => callback(err, pageData));
    }
}
exports.default = PriceFinder;
module.exports = PriceFinder;
