"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processPrice = exports.findContentOnPage = exports.categories = void 0;
const logger_1 = __importDefault(require("./logger"));
const accounting_1 = __importDefault(require("accounting"));
const lodash_1 = __importDefault(require("lodash"));
const isNumber = (value) => !Number.isNaN(parseFloat(value));
// Represents the various categories of an online item
exports.categories = Object.freeze({
    BOOKS: 'Books',
    CAMERA_VIDEO: 'Camera & Video',
    DIGITAL_MUSIC: 'Digital Music',
    ELECTRONICS: 'Electronics',
    HEALTH_PERSONAL_CARE: 'Health & Personal Care',
    HOME_AUDIO: 'Home Audio',
    HOUSEHOLD: 'Household',
    KINDLE_BOOKS: 'Kindle Books',
    LUGGAGE: 'Luggage',
    MOBILE: 'Mobile Phones',
    MOBILE_APPS: 'Mobile Apps',
    MOVIES_TV: 'Movies & TV',
    MUSIC: 'Music',
    OTHER: 'Other',
    TELEVISION_VIDEO: 'Television & Video',
    TOYS_GAMES: 'Toys & Games',
    VIDEO_GAMES: 'Video Games',
});
/**
 * Finds content on a page, returning either the text or null.
 *
 * @param  {any} $         jQuery-like object
 * @param  {Array<string>} selectors  An array of selectors to search with
 * @return {String | null}           The content found (or null)
 */
function findContentOnPage(
// TODO can we do better here than any?
// rome-ignore lint/suspicious/noExplicitAny: use any for now
$, selectors) {
    logger_1.default.debug('selectors: %j', selectors);
    // use the find() API in case we're handed an inner jquery object
    const jQuery = lodash_1.default.isFunction($.find) ? $.find.bind($) : $;
    // loop until we find the content, or we exhaust our selectors
    for (let i = 0; i < selectors.length; i++) {
        const content = jQuery(selectors[i]);
        if (!lodash_1.default.isEmpty(content)) {
            logger_1.default.debug('found content with selector: %s', selectors[i]);
            // if it's an array, return the first element
            if (content.length && content.length > 1) {
                logger_1.default.debug('content is an array, attempting to return first entry');
                return jQuery(selectors[i])
                    .first()
                    .text()
                    .trim();
            }
            else if (lodash_1.default.isFunction(content.text) && content.text().trim()) {
                // if we have text, return it
                return content.text().trim();
            }
            else {
                // nothing here, keep looking
            }
        }
    }
    // if we've not found anything, return null to signify that
    return null;
}
exports.findContentOnPage = findContentOnPage;
const DOLLAR_PREFIX = '$';
const DOLLAR_TEXT = 'usd';
const EURO_TEXT_PREFIX = 'eur';
const EURO_SYMBOL_PREFIX = '€';
const YEN_TEXT_PREFIX = '円';
const YEN_SYMBOL_PREFIX = '￥';
const GBP_SYMBOL_PREFIX = '£';
const GBP_TEXT_PREFIX = 'gbp';
const INR_SYMBOL_PREFIX = 'r';
const INR_TEXT_PREFIX = 'inr';
function processPrice(priceStringInput) {
    const priceString = priceStringInput.toLowerCase();
    logger_1.default.debug('price string (lowercased): %s', priceString);
    // currency specific processing
    let price;
    if (lodash_1.default.includes(priceString, DOLLAR_PREFIX) ||
        lodash_1.default.includes(priceString, DOLLAR_TEXT)) {
        logger_1.default.debug('found dollar in price, converting to number...');
        price = accounting_1.default.unformat(priceString);
    }
    else if (lodash_1.default.includes(priceString, EURO_TEXT_PREFIX) ||
        lodash_1.default.includes(priceString, EURO_SYMBOL_PREFIX)) {
        logger_1.default.debug('found euro in price, converting to number...');
        price = accounting_1.default.unformat(priceString, ',');
    }
    else if (lodash_1.default.includes(priceString, YEN_TEXT_PREFIX) ||
        lodash_1.default.includes(priceString, YEN_SYMBOL_PREFIX)) {
        logger_1.default.debug('found yen in price, converting to number...');
        price = accounting_1.default.unformat(priceString);
    }
    else if (lodash_1.default.includes(priceString, GBP_SYMBOL_PREFIX) ||
        lodash_1.default.includes(priceString, GBP_TEXT_PREFIX)) {
        logger_1.default.debug('found gbp in price, converting to number...');
        price = accounting_1.default.unformat(priceString);
    }
    else if (lodash_1.default.includes(priceString, INR_SYMBOL_PREFIX) ||
        lodash_1.default.includes(priceString, INR_TEXT_PREFIX)) {
        logger_1.default.debug('found inr in price, converting to number...');
        price = accounting_1.default.unformat(priceString);
    }
    else {
        // if no currency symbol was found, verify it is a number
        logger_1.default.debug('no currency symbol was found, verifying this is a number');
        if (!isNumber(priceString)) {
            logger_1.default.debug('price string is NOT a number, unable to process, returning -1');
            price = -1;
        }
        else {
            logger_1.default.debug('price string is a number, using defaults to convert...');
            price = accounting_1.default.unformat(priceString);
        }
    }
    logger_1.default.debug('price (post-process): %s', price);
    return price;
}
exports.processPrice = processPrice;
